% AX_DESIGN_NODES Finds the correct placements of nodes on impeller back shroud
% AX_DESIGN_NODES(R_1i,R_2,in_z_pos) 
% This function is used to determine the number of nodes in directions
% along and across the streamlines, as well as the positions of the nodes
% when they have been distibuted evenly along the streamlines.
% n      := number of nodes along the streamlines
% m      := number of nodes across the streamlines
% ax_pos := Position where the pump becomes completely radial as a portion
%    of D_2
% node   := Define the nodes
%     (# nodes along streamlines, first node is at outlet and last at inlet
%      # nodes across streamlines, first node is at .................
%      coordinates in r,theta and z)
% rad_pos := absolute position where the pump becomes completely raidal
% r_pos_ax_line := even distribution of points 
% z_pos_ax_line := even distribution of points
% xx := Distributing the r-position of nodes along the first streamline
% streamline_1_s := z-positions of the points along the spline with points
%                   evenly distributed along the r-axis
% new_r_pos := r-positions of the points made from a spline with points
%              evenly distributed along the first streamline
% new_z_pos := z-positions of the points made from a spline with points
%              evenly distributed along the first streamline
% d_nodes_lengths := length between nodes when distributed evenly along
%              r-axis (will most likely not be different between nodes)
% lengths := length between nodes when distributed evenly along streamline
%              (should be same all the way between all nodes)

function [node,r_pos,z_pos,new_r_pos,new_z_pos] = ...
    ax_design_nodes(n,m,ax_pos,D_2,D_1i,in_z_pos);

nodes_total  = n*m;
R_2          = 0.5*D_2;
R_2_red      = R_2/R_2; % Should be 1
R_1i        = 0.5*D_1i;
R_1i_red    = R_1i/R_2;
in_z_pos_red = in_z_pos/R_2;

% if (ax_pos <= R_1i_red || ax_pos >= 1)
%     fprintf('ERROR: Impeller "too" radial \n');
%     fprintf('Value should be between %u and 1 \n',(R_1i_red));
%     ax_pos = input('Pick a value between (R_1i/R_2) and 1: ');
%     if (ax_pos <= R_1i_red || ax_pos >= 1)
%         fprintf('Program will stop, wrong value selected, start program again \n');
%         return
%     end
% end

%%%%%% Define the nodes %%%%%%%%
% node( # of nodes along str. line ,  ... 
%       # of nodes across str. line, ...
%       (r,theta,z)-coordinates, ...
node = zeros(n,m,3);

% Setting the known values of nodes.
node(1,1,:) = [R_2 0 0]; % [r,theta,z] coordinates of node(1,1) first node on first streamline
node(n,1,:) = [R_1i 0 in_z_pos]; % (r,theta,z) coordinates of node(n,1) last node on first streamline
rad_pos = round(n*(1-ax_pos));

node(rad_pos,1,:) = [R_2*ax_pos 0 0]; % (r,theta,z) coordinates of some node between the first and last node on first streamline

if node(1,1,3)==node(rad_pos,1,3)
    r_pos_ax_line = linspace(node(1,1,1),node(rad_pos,1,1),rad_pos);
    z_pos_ax_line = linspace(node(1,1,3),node(rad_pos,1,3),rad_pos);
    r_pos = [r_pos_ax_line node(n,1,1)]; % getting the r-positions of node(1,1), node(n,1) and a node between
    z_pos = [z_pos_ax_line node(n,1,3)]; % getting the z-positions of node(1,1), node(n,1) and a node between
    clear r_pos_ax_line z_pos_ax_line;
else
    r_pos = [node(1,1,1) node(rad_pos,1,1) node(n,1,1)]; % getting the r-positions of node(1,1), node(n,1) and a node between
    z_pos = [node(1,1,3) node(rad_pos,1,3) node(n,1,3)]; % getting the z-positions of node(1,1), node(n,1) and a node between
end


xx = linspace(r_pos(1),r_pos(end),n); % Distributing the r-positions of nodes along the first streamline
% streamline_1_p = pchip(r_pos,z_pos,xx); % Hermite Interpolation
% streamline_1_s = interp1(r_pos,z_pos,xx,'spline')
streamline_param = spline(r_pos,z_pos);
streamline_1_s = ppval(streamline_param,xx); % Cubic Spline
node(:,1,1) = xx;
node(:,1,3) = streamline_1_s;

G = zeros(n,1);
d_r_pos = zeros(n-1,1);
d_z_pos = d_r_pos;

G(1,1) = 0;

for i=2:n
    R = node(:,1,1); 
    z = node(:,1,3); 
    G_temp = sqrt((R(i-1)-R(i))^2+(z(i-1)-z(i))^2);
    G(i) = G(i-1)+G_temp;
    d_r_pos(i-1) = R(i)-R(i-1);
    d_z_pos(i-1) = z(i)-z(i-1);
end

gg = linspace(min(G),max(G),n);
% Get a new positions of the nodes, with equal spaces between them
new_r_pos = spline(G,R,gg);
new_z_pos = spline(G,z,gg);
