% AX_DESIGN_NODES Finds the correct placements of nodes on impeller back shroud
% AX_DESIGN_NODES(R_1i,R_2,in_z_pos) 
% This function is used to determine the number of nodes in directions
% along and across the streamlines, as well as the positions of the nodes
% when they have been distibuted evenly along the streamlines.
% n      := number of nodes along the streamlines
% m      := number of nodes across the streamlines
% ax_pos := Position where the pump becomes completely radial as a portion
%    of D_2
% node   := Define the nodes
%     (# nodes along streamlines, first node is at outlet and last at inlet
%      # nodes across streamlines, first node is at .................
%      coordinates in r,theta and z)
% rad_pos := absolute position where the pump becomes completely raidal
% r_pos_ax_line := even distribution of points 
% z_pos_ax_line := even distribution of points
% xx := Distributing the r-position of nodes along the first streamline
% streamline_1_s := z-positions of the points along the spline with points
%                   evenly distributed along the r-axis
% new_r_pos := r-positions of the points made from a spline with points
%              evenly distributed along the first streamline
% new_z_pos := z-positions of the points made from a spline with points
%              evenly distributed along the first streamline
% d_nodes_lengths := length between nodes when distributed evenly along
%              r-axis (will most likely not be different between nodes)
% lengths := length between nodes when distributed evenly along streamline
%              (should be same all the way between all nodes)

function [node,r_pos,z_pos,new_r_pos,new_z_pos] = ...
    ax_design_nodes_diffuser(n,m,D_4,D_3,b_4,b_3,b_2,R_2);

nodes_total  = n*m;
R_4          = 0.5*D_4;
R_4_red      = R_4/R_2;
R_3        = 0.5*D_3;
R_3_red    = R_3/R_2;
b_4_red     = b_4/R_2;
b_3_red     = b_3/R_2;
b_2_red     = b_2/R_2;

%%%%%% Define the nodes %%%%%%%%
% node( # of nodes along str. line ,  ... 
%       # of nodes across str. line, ...
%       (r,theta,z)-coordinates, ...
node = zeros(n,m,3);

%Calculating the z-position of the known nodes.
z_pos_11    = b_4-(b_3-b_2)/2; 
z_pos_n1    = (b_3-b_2)/2;
% z_pos_1m    = -b_2-(b_3-b_2)/2;
% z_pos_nm    = -b_2-(b_3-b_2)/2;


% Setting the known values of nodes.
node(1,1,:) = [R_4 0 z_pos_11]; % [r,theta,z] coordinates of node(1,1) first node on first streamline
node(n,1,:) = [R_3 0 z_pos_n1]; % (r,theta,z) coordinates of node(n,1) last node on first streamline



%rad_pos = round(n*(1-ax_pos));

%node(rad_pos,1,:) = [R_2*ax_pos 0 0]; % (r,theta,z) coordinates of some node between the first and last node on first streamline


r_pos = [node(1,1,1) node(n,1,1)]; % getting the r-positions of node(1,1) and node(n,1) 
z_pos = [node(1,1,3) node(n,1,3)]; % getting the z-positions of node(1,1) and node(n,1) 



xx = linspace(R_4,R_3,n); % Distributing the r-positions of nodes along the first streamline
% streamline_1_p = pchip(r_pos,z_pos,xx); % Hermite Interpolation
% streamline_1_s = interp1(r_pos,z_pos,xx,'spline')
streamline_param = spline(r_pos,z_pos);
streamline_1_s = ppval(streamline_param,xx); % Cubic Spline
node(:,1,1) = xx;
node(:,1,3) = streamline_1_s;

G = zeros(n,1);
d_r_pos = zeros(n-1,1);
d_z_pos = d_r_pos;

G(1,1) = 0;

for i=2:n
    R = node(:,1,1); 
    z = node(:,1,3); 
    G_temp = sqrt((R(i-1)-R(i))^2+(z(i-1)-z(i))^2);
    G(i) = G(i-1)+G_temp;
    d_r_pos(i-1) = R(i)-R(i-1);
    d_z_pos(i-1) = z(i)-z(i-1);
end

gg = linspace(min(G),max(G),n);
% Get a new positions of the nodes, with equal spaces between them
new_r_pos = spline(G,R,gg);
new_z_pos = spline(G,z,gg);
