% STREAMLINES Defines the streamlines of the impeller based
% STREAMLINES(n,m,node,D_2,b_2,D_1,D_1i,A_1,A_2,A2_A1)
% This function is used to find the streamlines in the meridional plane, 
% first streamline is already found, the back shroud of the impeller, 
% from that the other streamlines can be found since the area 
% between parallell streamline is equal 
% out_area_st := the area at outlet divided into m-1 spaces
% dr_temp     := delta r used to find the normal vector
% dz_temp     := delta z used to find the normal vector
% l           := length of the normal vector
% unit_dr     := the delta r length as a unit vector (length as 1)
% unit_dz     := the delta z length as a unit vector (length as 1)
% dl          := total length of the normal vector

function [r,z,dr_temp,dz_temp,unit_dr,unit_dz,l,dA_n] = ...
    ax_design_streamlines(n,m,node,D_2,b_2,D_1,D_1i,A_1,A_2,A2_A1)

R_2       = 0.5*D_2;
R_2_red   = R_2/R_2; % Defined as 1
b_2_red   = b_2/R_2;
D_1_red   = D_1/R_2;
R_1i     = 0.5*D_1i;
D_1i_red = D_1i/R_2;
R_1i_red = 0.5*D_1i_red;

% Setting r and z for better understanding code.
r = node(:,:,1);
z = node(:,:,3);

% Setting the positions of the nodes along the outlet (1,1:m,:)
r(1,m) = r(1,1);
z(1,m) = z(1,1) + b_2; % since the impeller outlet is radial
r(1,:) = linspace(r(1,1),r(1,m),m);
z(1,:) = linspace(z(1,1),z(1,m),m);
dA_2 = A_2/(m-1);
dA_n = linspace(A_2,A_1,n); dA_n = dA_n';
% These parameter are used for finding the normal vector out of the 
% streamlines as they are found.
dr_temp = zeros(n,m);
dz_temp = zeros(n,m);
l = zeros(n,m);
unit_dr = zeros(n,m);
unit_dz = zeros(n,m);
dl = zeros(n,1);

for j=1:m-1
    dr_temp(1,j) = r(2,j) - r(1,j);
    dz_temp(1,j) = z(2,j) - z(1,j);
    l(1,j) = sqrt(dr_temp(1,j)^2+dz_temp(1,j)^2);
    unit_dr(1,j) = dr_temp(1,j)/l(1,j);
    unit_dz(1,j) = dz_temp(1,j)/l(1,j);
    for i=2:n-1
        dr_temp(i,j) = 0.5*(r(i+1,j)-r(i-1,j));
        dz_temp(i,j) = 0.5*(z(i+1,j)-z(i-1,j));
        l(i,j) = sqrt(dr_temp(i,j)^2+dz_temp(i,j)^2);
        unit_dr(i,j) = dr_temp(i,j)/l(i,j);
        unit_dz(i,j) = dz_temp(i,j)/l(i,j); 
    end
    dr_temp(n,j) = r(n,j)-r(n-1,j);
    dz_temp(n,j) = z(n,j)-z(n-1,j);
    l(n,j) = sqrt(dr_temp(n,j)^2+dz_temp(n,j)^2);
    unit_dr(n,j) = dr_temp(n,j)/l(n,j);
    unit_dz(n,j) = dz_temp(n,j)/l(n,j);
    for i=2:n
        A = 1/(2*sqrt(unit_dr(i,j)^2+unit_dz(i,j)^2)); % parameter used to find dl
        B = r(i,j); % parameter used to find dl
        C = -dA_n(i)/(2*pi*(m-1)); % parameter used to find dl
        dl(i,j) = (-B+sqrt(B^2-4*A*C))/(2*A);
        r(i,j+1) = r(i,j)+unit_dz(i,j)*dl(i,j);
        z(i,j+1) = z(i,j)-unit_dr(i,j)*dl(i,j);
    end

end

return