function change_line(h,opt,tool)
% function change_line(h,option)
%
% INPUTS:      h = handle to a line object (e.g. created by a plot command)
%         option = [  'x' | 'y' | {'xy'} | 'axy' ]
%
% This function, once enabled, allows the user to manipulate the coordinates of any
% point in a line object specified by handle h, simply by clicking and moving the mouse.
%
% If the function is invoked with no option, or with the option 'xy', then clicking the mouse 
% over the line will allow the user to drag the point to a new x- and y- coordinate, as long 
% as the new x coordinate is between the two adjacent points.  If the function is invoked 
% with the option 'x' or 'y' then the point will only move in the specified coordinate, 
% leaving the other coordinate fixed. 
%
% If the function is called with the option 'x' or with the option 'axy', the x-coordinate 
% will not be limited to values between adjacent points.
%
% When the button is released, the point will take on the new values.
%
% EXAMPLE:
% >> h = plot(x, sin(x));
% >> change_line(h,'xy')
% >>
%
% If the function is invoked on a line which already uses the change_line function, then the
% function will be disabled.  If the function is called on a line with z-data, then the
% function will return an error and not be invoked.
%
% The function can also be disabled by calling it with the opt argument equal to 'off'.
%
% EXAMPLE:
% >> change_line(h,'off')
% >>
%
% This function makes use of the current axes' 'userdata' property, which must be made available
% to it.  During the immediate execution of the function (i.e. as long as the button is being held down), 
% the axes' 'userdata' property is unavailable; however, upon the release of the mouse button, the
% function restores the axes' userdata to its initial value.
%
% Written by Brandon Kuczenski for Kensington Labs
% brandon_kuczenski@kensingtonlabs.com
% 17 September 2001
%
% Modified to restore button functions to original state
% And test for double-click to end change_line action
% Alex Woo, woo@geomi.com
% 26 February 2002
%

declare_global_variables();

switch nargin
case 0
    error('Must Specify a handle to a line object')
case 1
    opt='xy';
case 2
    if ~any(strcmp(opt,{'x','y','xy','axy','a','off'}))
        error(['Second argument ''' opt ''' is an unknown command option.'])
    end
end

if nargin<3  % i.e. user-invoked.  
    b=get(h,'tag');
    if ~isempty(b)&strcmp(b,'change_line')&(nargin==1|strcmp(opt,'off'))  % function being de-invoked
        k=get(h,'userdata');
        %  acw modified to recall buttondownfcn and restore when turned off
        set(h,'buttondownfcn',k.bdfcn,'markersize',k.ms,'marker',k.m,'markerfacecolor',k.mfc,'userdata','','tag','')
    else % function being invoked
        if ~isempty(get(h,'zdata'))
            error('change_line only works for 2-D graphs')
        end
        if strcmp(opt,'a')
            opt='axy';
        end
        if isempty(b) % only assign original values to marker struct if function is being called the first time
            k.m=get(h,'marker');set(h,'marker','o');
            k.ms=get(h,'markersize');set(h,'markersize',4);
            k.mfc=get(h,'markerfacecolor');set(h,'markerfacecolor',get(h,'color'));
        else
            k=get(h,'userdata');
        end
        k.opt=opt;
        k.bdfcn=get(h,'buttondownfcn'); % save the current buttondownfcn before reset
        set(h,'buttondownfcn',['change_line(gco,''' opt ''',1)'],'userdata',k,'tag','change_line');
        set(findobj('children',h),'units','pixels')
    end
else  % i.e. self-invoked
    if strcmp(get(gcf,'selectiontype'),'open')
        change_line(h,'off');
    else
        switch tool
        case 1 % line's buttondownfcn invoked
            if ~isempty(get(h,'zdata'))
                change_line(h)
                error('change_line only works for 2-D graphs')
            end
            cp=get(gca,'currentpoint');
            k=get(h,'userdata');
            y=abs(get(h,'ydata')-cp(2,2));x=abs(get(h,'xdata')-cp(1,1)); % determine which point the user clicked on
            % just use the distance function
            d=sqrt(y.^2+x.^2);
            k.index=find(d==min(d));
            k.axesdata=get(gca,'userdata');
            k.doublebuffer=get(gcf,'doublebuffer');
            k.winbmfcn = get(gcf,'windowbuttonmotionfcn');  %  save current window motion function
            k.winbupfcn = get(gcf,'windowbuttonupfcn');  %  save current window up function
            k.winbdfcn = get(gcf,'windowbuttondownfcn');  %  save current window down function
            
            set(h,'userdata',k);
            set(gcf,'windowbuttonmotionfcn',['change_line(get(gca,''userdata''),''' opt ''',2)'],'doublebuffer','on');
            set(gca,'userdata',h);
            set(gcf,'windowbuttonupfcn',['change_line(get(gca,''userdata''),''' opt ''',3)']);
            
            %         end    
        case 2 % button motion function
            k=get(h,'userdata');
            cp=get(gca,'currentpoint');
            x=get(h,'xdata');
            y=get(h,'ydata');
            switch opt
            case 'x'
                x(k.index)=cp(1,1);
            case 'y'
                y(k.index)=cp(2,2);
            case {'xy','axy'}
                x(k.index)=cp(1,1);
                y(k.index)=cp(2,2);
            end
            
            if (~strcmp(opt,'axy')&~strcmp(opt,'x'))
                if k.index>1&x(k.index)<x(k.index-1)
                    x(k.index)=x(k.index-1);
                end
                if k.index<length(x)&x(k.index)>x(k.index+1)
                    x(k.index)=x(k.index+1);
                end
            end
            
            set(h,'xdata',x,'ydata',y)
            % test to see if it moved off the screen - update limits
            fgx=get(gca,'xlim');
            fgy=get(gca,'ylim');
            if any(opt=='y')&cp(2,2)>fgy(2)
                set(gca,'ylim',[fgy(1) cp(2,2)])
            end
            if any(opt=='y')&cp(2,2)<fgy(1)
                set(gca,'ylim',[cp(2,2) fgy(2)])
            end
            if any(opt=='x')&cp(1,1)>fgx(2)
                set(gca,'xlim',[fgx(1) cp(1,1)])
            end
            if any(opt=='x')&cp(1,1)<fgx(1)
                set(gca,'xlim',[cp(1,1) fgx(2)])
            end
            
        case 3 % button up - we're done
            
            % Update the axial cut plot

            Px=get(h,'Xdata');
            Py=get(h,'Ydata');
            
            % Making the inlet and outlet points unable to be dragged 
            % with a mouse on the plot
            
            D_2 = valg_design_D2;
            R_2 = 0.5*D_2; R_2_red = R_2/R_2;
            R_1i = 0.5*valg_design_D1i;
            Px(end) = R_1i/R_2; % inlet point r-position
            Px(1)   = R_2/R_2; % outlet point r-position
            Py(end) = valg_design_in_z_pos/R_2;
            Py(1)   = 0;
            set(h,'XData',Px,'YData',Py);
            param_bezier_ax_cut_x = Px;
            param_bezier_ax_cut_y = Py;

            
            valg_design_bezier_x=Px; valg_design_bezier_y=Py;
            [Qx,Qy]=Funct_Bezier_unplot(Px,Py,valg_npoints_cross-1);

            [Qx,Qy] = even_spaces(Qx,Qy,valg_npoints_cross);
            valg_ax_nodes(:,1,1) = R_2*Qx;
            valg_ax_nodes(:,1,3) = R_2*Qy;
            
            % Calculate the streamlines
            [valg_ax_nodes(:,:,1),valg_ax_nodes(:,:,3),param_ax_view_dr, ...
              param_ax_view_dz,unit_dr,unit_dz,normal_length,dA_n] = ...
                ax_design_streamlines( ...
                    valg_npoints_cross, ...
                    valg_npoints_stream, ...
                    valg_ax_nodes, ...
                    valg_design_D2, ...
                    valg_design_b2, ...
                    get(hand_calc_D1,'Value'), ...
                    valg_design_D1i, ...
                    param_A1, ...
                    param_A2, ...
                    hand_calc_ratA ...
                );
            valg_ax_nodes_red(:,:,1) = valg_ax_nodes(:,:,1)/(R_2);
            valg_ax_nodes_red(:,:,3) = valg_ax_nodes(:,:,3)/(R_2);

            
            
            for i=1:valg_npoints_stream; 
                set(hand_plot_ax_stream_line(i),'XData',valg_ax_nodes_red(:,i,1),'YData',valg_ax_nodes_red(:,i,3));
            end;
            for i=1:valg_npoints_cross; 
                set(hand_plot_ax_cross_line(i),'XData',valg_ax_nodes_red(i,:,1),'YData',valg_ax_nodes_red(i,:,3));
            end;

            % Setting the axis values automatic
            r_temp_min = [min(min(Px)) min(min(valg_ax_nodes_red(:,:,1)))];
            r_temp_max = [max(max(Px)) max(max(valg_ax_nodes_red(:,:,1)))];
            z_temp_min = [min(min(Py)) min(min(valg_ax_nodes_red(:,:,3)))];
            z_temp_max = [max(max(Py)) max(max(valg_ax_nodes_red(:,:,3)))];
            r_min = min(r_temp_min);
            r_max = max(r_temp_max);
            z_min = min(z_temp_min);
            z_max = max(z_temp_max);
            r_axis_adjust = 0.1*abs(r_max);
            z_axis_adjust = 0.1*abs(z_max-z_min);
            if r_min>0; r_min=0; end;
            set(gca,'XLim',[r_min r_max+r_axis_adjust], ...
                    'YLim',[z_min-z_axis_adjust z_max+z_axis_adjust]);
            
            k=get(h,'userdata');
            set(gca,'userdata',k.axesdata); % restore axes data to its previous value
            set(gcf,'windowbuttonmotionfcn',k.winbmfcn,'windowbuttonupfcn',k.winbupfcn,'doublebuffer',k.doublebuffer)
            
            
            
            
        end
    end
end
